import logging
from typing import Optional

from dmsales.api_operations import APIOperations

logger = logging.getLogger(__name__)

class UserEndpoints(APIOperations):
    
    def me(self) -> Optional[dict]:
        '''
        Returns user information

        :return: json response from api
        :rtype: Optional[dict]
        '''
        return super().make_get_request(endpoint='/api/user/me')

    def my_points(self) -> Optional[dict]:
        '''
        Returns user points

        :return: json response from api
        :rtype: Optional[dict]
        '''
        return super().make_get_request(endpoint='/api/user/wallet/points')



import logging

import requests.exceptions

from . import exceptions

logger = logging.getLogger(__name__)
DEFAULT_TIMEOUT = 10
class APIOperations:
    '''
    Handling requests
    '''
    
    def make_get_request(self, endpoint: str, **kwargs):
        '''
        Sends GET request to given endpoint and handle errors

        :param endpoint: only endpoint e.g. /api/persons/list
        :type endpoint: str
        :raises exceptions.DMSalesAPIConnectionError: when there was a problem with api connection (timeouts) 
        :raises Exception: unknown error
        :return: json response from api
        :rtype: dict
        '''
        try:
            logger.debug(f'Trying to make GET request to endpoint {endpoint} with query kwargs {kwargs}')
            if 'timeout' in kwargs:
                response = self.session.get(self.api_host + endpoint, **kwargs)
            else:
                response = self.session.get(self.api_host + endpoint, timeout=DEFAULT_TIMEOUT, **kwargs)
        except (requests.exceptions.ConnectTimeout,
                requests.exceptions.ReadTimeout) as e:
            logger.exception('DMSales API connection timeout')
            raise exceptions.DMSalesAPIConnectionError(DEFAULT_TIMEOUT)
        except Exception as e:
            raise e
        else:
            logger.debug(f'DMSales API returned response {response}')
            return response.json()

    def make_post_request(self, endpoint: str, **kwargs):
        '''
        Sends POST request to given endpoint and handle errors

        :param endpoint: only endpoint e.g. /api/persons/list
        :type endpoint: str
        :raises exceptions.DMSalesAPIConnectionError: when there was a problem with api connection (timeouts) 
        :raises Exception: unknown error
        :return: json response from api
        :rtype: dict
        '''
        try:
            logger.debug(f'Trying to make POST request to endpoint {endpoint} with kwargs {kwargs}')
            if 'timeout' in kwargs:
                response = self.session.post(self.api_host + endpoint, **kwargs)
            else:
                response = self.session.post(self.api_host + endpoint, timeout=DEFAULT_TIMEOUT, **kwargs)
        except (requests.exceptions.ConnectTimeout,
                requests.exceptions.ReadTimeout) as e:
            raise exceptions.DMSalesAPIConnectionError(DEFAULT_TIMEOUT)
        except Exception as e:
            raise e
        else:
            logger.debug(f'DMSales API returned response {response}')
            logger.debug(f'Response message: {response.text}')
            return response.json()

    def make_put_request(self, endpoint: str, **kwargs):
        '''
        Sends PUT request to given endpoint and handle errors

        :param endpoint: only endpoint e.g. /api/persons/list
        :type endpoint: str
        :raises exceptions.DMSalesAPIConnectionError: when there was a problem with api connection (timeouts) 
        :raises Exception: unknown error
        :return: json response from api
        :rtype: dict
        '''
        try:
            logger.debug(f'Trying to make PUT request to endpoint {endpoint} with kwargs {kwargs}')
            if 'timeout' in kwargs:
                response = self.session.put(self.api_host + endpoint, **kwargs)
            else:
                response = self.session.put(self.api_host + endpoint, timeout=DEFAULT_TIMEOUT, **kwargs)
        except (requests.exceptions.ConnectTimeout,
                requests.exceptions.ReadTimeout) as e:
            raise exceptions.DMSalesAPIConnectionError(DEFAULT_TIMEOUT)
        except Exception as e:
            raise e
        else:
            logger.debug(f'DMSales API returned response {response}')
            logger.debug(f'Response message: {response.text}')
            return response.json()



import logging
import json
from typing import Optional

from dmsales.api_operations import APIOperations

logger = logging.getLogger(__name__)


class EventsEndpoints(APIOperations):

    def add_custom_event(self, project_id: str, type: str, custom: dict, base_key: str = None,
                         email: str = None, phone: str = None, public_identifier_li: str = None) -> Optional[str]:
        '''
        Adds custom event.
        One arg of (email, base_key, phone, public_identifier_li) is required.

        :param project_id: project id e.g. 123e4567-e89b-12d3-a456-426614174000
        :type project_id: str
        :param type: event type e.g. custom_type
        :type type: str
        :param custom: custom fields and values added to custom event
        :type custom: dict
        :param base_key: contact base key in project e.g. bazaMR_1234567890, defaults to None
        :type base_key: str, optional
        :param email: contact email in project, defaults to None
        :type email: str, optional
        :param phone: contact phone in project, defaults to None
        :type phone: str, optional
        :param public_identifier_li: contact public linkedin identifier in project e.g. imie-nazwisko-123456789, defaults to None
        :type public_identifier_li: str, optional
        :return: message response from endpoint (e.g. "ok" or "Project person not found")
        :rtype: Optional[str]
        '''
        endpoint = '/api/events/add-custom-event'

        data = {
            'project_uuid': project_id,
            'type': type,
            'base_key': base_key,
            'email': email,
            'phone': phone,
            'public_identifier_li': public_identifier_li,
            'custom': custom
        }

        data = {k: v for k, v in data.items() if v}  # exclude None args
        logger.debug('Calling add_custom_event method')
        return super().make_post_request(endpoint, json=data)



import logging
from typing import Optional

from dmsales.api_operations import APIOperations

logger = logging.getLogger(__name__)

import requests

class TechScopeApiEndpoints(APIOperations):
    
    def start_techscopeapi(self, www: str) -> Optional[dict]:
        '''
        Starts techscopeapi generation process

        :param www: WWW to start techscopeapi
        :type www: str
        :return: json response from api
        :rtype: Optional[dict]
        '''

        return super().make_post_request(endpoint='/api/techscopeapi/generate', params={'www': www})
        
    
    def techscopeapi_result(self, task_id: str) -> Optional[dict]:
        '''
        Get techscopeapi result with state

        :param task_id: task ID to check result (from start_techscopeapi method)
        :type task_id: str
        :return: json response from api
        :rtype: Optional[dict]
        '''
        return super().make_get_request(endpoint='/api/techscopeapi/result', params={'task_id': task_id})
        


import requests
import logging

from .endpoints import contacts, project, segment, events, search, person, user, techscopeapi

logger = logging.getLogger(__name__)

class DMSalesAPI(
    project.ProjectEndpoints,
    contacts.ContactsEndpoints,
    segment.SegmentEndpoints,
    events.EventsEndpoints,
    search.SearchEndpoints,
    person.PersonEndpoints,
    user.UserEndpoints,
    techscopeapi.TechScopeApiEndpoints
):
    
    api_host = 'https://app.dmsales.com'

    def __init__(self, api_key: str, test: bool=False):
        '''
        Main class to manipulate DMSales API with all implemented methods

        :param api_key: dmsales api key https://app.dmsales.com/pl/panel/settings-account?settings=api-configuration
        :type api_key: str
        :param test: dmsales api test environment, defaults to False
        :type test: bool, optional
        '''
        self.api_key = api_key

        if test is True:
            self.api_host = 'http://dmsales.test.dmsales.com:8081'

        self.session = requests.Session()
        self.session.headers = {'Authorization': f'Bearer {api_key}'}



import logging
from typing import Optional, List

from dmsales.api_operations import APIOperations

logger = logging.getLogger(__name__)

class ProjectEndpoints(APIOperations):
    
    def project_list(self) -> Optional[List[dict]]:
        '''
        This call returns user's all projects.

        :return: list of project dictionaries
        :rtype: Optional[List[dict]]
        '''
        endpoint = '/api/project/list'
        logger.debug('Calling project_list method')
        return super().make_get_request(endpoint)


import logging
from typing import List, Dict

from dmsales.api_operations import APIOperations

logger = logging.getLogger(__name__)

class SearchEndpoints(APIOperations):

    def search_list(self, project_id: str) -> List[Dict[str, str]]:
        '''
        This call return profiles and segments from project

        :param project_id: ID from projects' list
        :type project_id: str
        :return: List of dicts with profiles and segments from project
        :rtype: List[Dict[str, str]]
        '''
        return super().make_get_request('/api/search/list', params={'project_id': project_id})



import logging
from typing import Union, Dict, List, Optional
try:
    from typing import TypedDict
except ImportError:
    from typing_extensions import TypedDict

from dmsales.api_operations import APIOperations

logger = logging.getLogger(__name__)


class ProjectPerson(TypedDict):
    tags: List[str]
    type: str
    address: Dict[str, str]
    postal_code: str
    street: str
    street_number: str
    local_number: str
    voivodeship: str
    personal_data: Dict[str, str]
    email: Dict[str, str]
    phone: Dict[str, str]
    '''
    Example Dict:
    {
        "id": "bazaMR_0000000000",
        "project_id": "b5faf3c9-2672-4cf0-b0aa-57ce8c2a8a21",
        "project_person": {
            "tags": [
            "tag1",
            "tag2"
            ],
            "type": "b2b",
            "address": {
            "city": "Katowice",
            "county": "Katowice",
            "postal_code": "40-101",
            "street": "Al. Piastów",
            "street_number": "10",
            "local_number": "35",
            "voivodeship": "śląskie"
            },
            "personal_data": {
            "name": "Nowy",
            "surname": "Lead",
            "company_name": "My Very First Company",
            "position": "CEO"
            },
            "email": {
            "raw": "exae-email@test.test"
            },
            "phone": {
            "raw": "+48 123 456 678"
            },
            "sex": "kobieta"
        }
    }
    '''


class PersonEndpoints(APIOperations):

    def add_person(self, id: Optional[str], project_id: str, person_dict: ProjectPerson) -> Optional[dict]:
        '''
        Adds person to project

        :param id: person id
        :type id: Optional[str, None]
        :param project_id: project id where person will be added
        :type project_id: str
        :param person_dict: person data
        :type person_dict: ProjectPerson
        '''
        endpoint = '/api/persons/upsert'
        data = {
            'id': id,
            'project_id': project_id,
            'project_person': person_dict
        }
        return super().make_post_request(endpoint=endpoint, json=data)

    def update_person(self, id: str, project_id: str, person_dict: ProjectPerson) -> Optional[dict]:
        '''
        Updates person in project

        :param id: person id
        :type id: str
        :param project_id: project id where person will be updated
        :type project_id: str
        :param person_dict: person_data
        :type person_dict: ProjectPerson
        '''
        endpoint = '/api/persons/upsert'
        data = {
            'id': id,
            'project_id': project_id,
            'project_person': person_dict
        }
        return super().make_put_request(endpoint=endpoint, json=data)

    def init_call(self, project_id: str, base_key: str) -> Optional[dict]:
        '''
        Initializes call to person from project

        :param project_id: project id
        :type project_id: str
        :param base_key: person's base key
        :type base_key: str
        '''
        params = {'project_id': project_id, 'base_key': base_key}
        return super().make_post_request(endpoint='/api/persons/init-call', params=params)

    def check_call(self, project_id: str, call_id: str) -> Optional[dict]:
        '''
        Checks call status

        :param project_id: project id
        :type project_id: str
        :param call_id: call id which status you want to check
        :type call_id: str
        '''
        params = {'project_id': project_id, 'call_id': call_id}
        return super().make_get_request(endpoint='/api/persons/check-call', params=params)



class DMSalesAPIConnectionError(Exception):
    
    def __init__(self, timeout, *args, **kwargs):
        self.timeout = timeout
        super().__init__(*args, **kwargs)

    def __str__(self):
        return f'Connection Timeout (timeout: {self.timeout}). Check if your api key hasn\'t expired'




from setuptools import setup, find_packages

setup(
    name='dmsales',
    version='0.2.0',
    description='DMSales API Python Client',
    long_description='DMSales API Python Client',
    url='https://app.dmsales.com/api-doc/default',
    author='DMSales',
    author_email='data@dmsales.com',
    packages=find_packages(),
    install_requires=[
        'requests==2.25.1',
        'typing-extensions==3.10.0.0'
    ]
)



import logging
from typing import Optional

from dmsales.api_operations import APIOperations

logger = logging.getLogger(__name__)


class ContactsEndpoints(APIOperations):

    def persons_list(self, page: int, limit: int, segment_id: str = None,
                     project_id: str = None, sort: str = None, export_date_from: str = None,
                     export_date_to: str = None, paid_leads='true') -> Optional[dict]:
        '''
        This endpoint returns your project's contacts. You can filter by project or profile. 
        Additionaly, you can provide export dates to filter contacts from given period.

        :param page: Page number
        :type page: int
        :param limit: Amount of contacts per page
        :type limit: int
        :param segment_id: Segment ID - returns only contacts matching given profile. One of segment_id or project_id is required.
        :type segment_id: str
        :param project_id: Returns all contacts from given project. One of segment_id or project_id is required.
        :type project_id: str
        :param sort: Define list sorting.
        :type sort: str
        :param export_date_from: Filter contacts by export date (exported after given date).
        :type export_date_from: str
        :param export_date_to: Filter contacts by export date (exported before given date).
        :type export_date_to: str
        :param paid_leads: Display paid leads. (Available values : 'true', 'false', 'all'), defaults to 'true'
        :type paid_leads: str
        '''
        endpoint = '/api/persons/list'
        args_dict = {
            'page': page,
            'limit': limit,
            'segment_id': segment_id,
            'project_id': project_id,
            'sort': sort,
            'export_date_from': export_date_from,
            'export_date_to': export_date_to,
            'paid_leads': paid_leads
        }

        # exclude None args
        args_dict = {k: v for k, v in args_dict.items() if v}
        logger.debug('Calling persons_list method')
        return super().make_get_request(endpoint, params=args_dict)



import logging
from typing import Optional, List

from dmsales.api_operations import APIOperations

logger = logging.getLogger(__name__)


class SegmentEndpoints(APIOperations):

    def segment_list(self, project_id: str) -> Optional[List[dict]]:
        '''
        This call return segments from project

        :param project_id: ID from projects' list
        :type project_id: str
        :return: segment list
        :rtype: Optional[List[dict]]
        '''
        endpoint = '/api/segment/list'
        args_dict = {
            'project_id': project_id
        }
        return super().make_get_request(endpoint, params=args_dict)
        
    


import logging

import dmsales

logger = logging.getLogger(__name__)

def person_generator(dmsales_api: dmsales.DMSalesAPI, segment_id: int):
    '''
    Generator which generates all persons from specific segment

    :param dmsales_api: DMSalesAPI client object
    :type dmsales_api: dmsales.DMSalesAPI
    :param segment_id: segment id from persons will be downloaded
    :type segment_id: int
    :yield: yields one person object
    :rtype: None
    '''
    logger.debug(f'Getting persons list from segment {segment_id}')
    page = 1
    while True:
        leads_result = dmsales_api.persons_list(page=page, limit=100, segment_id=segment_id) # 10000 max limit on page
        
        logger.debug(f'Got persons_list result {leads_result}')
        if not leads_result or not isinstance(leads_result, dict):
            if 'maximum limit' in leads_result:
                logger.debug(f'Api returned max limit communicate: {leads_result}')
            return
        else:
            if len(leads_result['data']) == 0:
                logger.debug(f'No more leads from segment {segment_id}')
                return
            else:
                logger.debug(f'Downloaded persons list from segment {segment_id}')
                page += 1
                for lead in leads_result['data']:
                    yield lead